# Copyright (c) 2012-2015, 2019 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

from collections import OrderedDict
import datetime

import wdns

def iso8601now():
    return datetime.datetime.utcnow().isoformat() + 'Z'

def form_basic_log(env, logtype):
    ''' Forms a basic dnstli log message as an ordered dict.
        After any other fields are added to the dict, log it with: log(NOTICE, json.dumps(logDict))
    '''
    logDict = OrderedDict()
    logDict['timestamp'] = iso8601now()
    logDict['logtype'] = logtype
    logDict['host'] = env.get('SERVER_NAME', 'unset_SERVER_NAME')
    logDict['authtype'] = env.get('flexapiserver.authtype', 'unset_authtype')
    logDict['authuser'] = env.get('flexapiserver.authuser', 'unset_authuser')
    logDict['ip'] = env.get('REMOTE_ADDR', 'unset_REMOTE_ADDR')
    if env.get('flexapiserver.blkq_split'):
        logDict['blkq'] = True

    return logDict

# Convert string form rrtype names into their numeric equivalents
# and put those into a hash.  later, search the hash by numeric value.
# This allows us to support TYPE# equivalents.
allowed_rrtypes_for_rdata_searches_names = (
    'ANY',
    'SOA',
    'CNAME',
    'HINFO',
    'MX',
    'NS',
    'NAPTR',
    'PTR',
    'RP',
    'SRV',
    'SPF',
    'TXT' )

allowed_rrtypes_for_rdata_searches = set()

for rrtype in allowed_rrtypes_for_rdata_searches_names:
    allowed_rrtypes_for_rdata_searches.add(rrtype)
    allowed_rrtypes_for_rdata_searches.add(wdns.str_to_rrtype(rrtype))

dnssec_rrtypes_names = (
    # 'ANY-DNSSEC', # not official -- handle specially
    'DS',
    'RRSIG',
    'NSEC',
    'DNSKEY',
    'NSEC3',
    'NSEC3PARAM',
    'DLV',
    'CDNSKEY',
    'CDS' )

dnssec_rrtypes = set()

for rrtype in dnssec_rrtypes_names:
    dnssec_rrtypes.add(rrtype)
    dnssec_rrtypes.add(wdns.str_to_rrtype(rrtype))

def forbidden_rtype_for_rdata(rrtype_num):
    ''' Return True if the rrtype is forbidden for rdata searches; else False
    '''
    if rrtype:
        return rrtype_num not in allowed_rrtypes_for_rdata_searches
    else:
        return False

def is_dnssec(rrtype_num):
    ''' Return True if the rrtype is one of the DNSSec rrtypes; else False
    '''
    if rrtype:
        return rrtype_num in dnssec_rrtypes
    else:
        return False
