# Copyright (c) 2012-2018, 2020-2021 Farsight Security, Inc.
# All Rights Reserved
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF FARSIGHT SECURITY, INC.
# The copyright notice above does not evidence any
# actual or intended publication of such source code.

import distutils.version
import os
import sys
import time
import traceback

import uwsgi
from uwsgidecorators import postfork, timer

import flexapiserver.client
import flexapiserver.conf
import flexapiserver.log
import flexapiserver.wsgi.api
import flexapiserver.wsgi.auth
import flexapiserver.wsgi.cors

# Load the readers
try:
    config = flexapiserver.conf.get_config()
    client = flexapiserver.client.Client(config)
except Exception as e:
    tb = traceback.format_exc()
    print("flexapiserver FATAL error in getting and applying config: ", str(e), str(tb))
    sys.exit(1)


@postfork
def open_syslog():
    # Open syslog
    flexapiserver.log.openlog('flexapiserver')

# apikey references an anonymous function returned from the cors
# module.  It is the "module" name, aka WSGI application entry point,
# i.e., it is referenced from uWSGI configuration, so it should not be
# changed without coordination.  The variable apikey is only created
# if its preconditions are satisfied.  Exactly one of the functions
# will be called by invoking uwsgi with the corresponding "module"
# name.
assert os.path.isdir(flexapiserver.conf.API_DIR)
apikey = flexapiserver.wsgi.cors.cors(flexapiserver.wsgi.auth.apikey(flexapiserver.wsgi.api.FLEXAPI(client)))
